{
  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/release-24.05";

    flake-utils.url = "github:numtide/flake-utils";

    crane = {
      url = "github:ipetkov/crane";
      inputs.nixpkgs.follows = "nixpkgs";
    };
  };

  outputs =
    {
      self,
      nixpkgs,
      flake-utils,
      crane,
      ...
    }:
    flake-utils.lib.eachDefaultSystem (
      system:
      let
        pkgs = import nixpkgs { inherit system; };

        craneLib = crane.mkLib pkgs;

        commonArgs = {
          src = craneLib.cleanCargoSource ./.;
          strictDeps = true;
          nativeBuildInputs = with pkgs; [ pkg-config ];
          buildInputs = with pkgs; [ openssl ];
        };

        cargoArtifacts = craneLib.buildDepsOnly commonArgs;

        my-crate = craneLib.buildPackage (
          commonArgs
          // {
            src = ./.; # Allow access to assets, like ./templates
            inherit cargoArtifacts;
            meta.mainProgram = "made-you-look";
          }
        );
      in
      {
        checks = {
          inherit my-crate;
          my-crate-fmt = craneLib.cargoFmt { inherit (commonArgs) src; };
        };

        packages.default = my-crate;

        devShells.default = pkgs.mkShell {
          inherit (commonArgs) buildInputs;

          nativeBuildInputs =
            with pkgs;
            (
              [
                rustc
                cargo
                rustfmt
                rust-analyzer
                clippy
                cargo-feature
                cargo-watch
                curl
              ]
              ++ commonArgs.nativeBuildInputs
            );
        };
      }
    )
    // {

      nixosModules.default =
        {
          pkgs,
          lib,
          config,
          ...
        }:
        let
          cfg = config.services.made-you-look;
        in
        {
          options.services.made-you-look = {
            enable = lib.mkEnableOption "Enable Warthunder Leak Counter";

            port = lib.mkOption {
              type = lib.types.port;
              default = 26023;
            };
          };
          config = lib.mkIf cfg.enable {
            systemd.services.made-you-look = {
              script = lib.getExe self.packages.${pkgs.system}.default;

              environment = {
                MADE_YOU_LOOK_SERVE_PORT = toString cfg.port;
              };

              wantedBy = [ "multi-user.target" ];
              after = [ "network.target" ];
            };
          };
        };

      nixosConfigurations.test-server = nixpkgs.lib.nixosSystem {
        system = "x86_64-linux";
        modules = [
          self.nixosModules.default
          (nixpkgs + "/nixos/modules/virtualisation/qemu-vm.nix")
          (
            { config, ... }:
            {
              services.made-you-look.enable = true;
              users.users.root.password = "root";
              networking.firewall.enable = false;
              virtualisation.forwardPorts = [
                {
                  from = "host";
                  host.port = 8888;
                  guest.port = config.services.made-you-look.port;
                }
              ];
              system.stateVersion = "24.05";
            }
          )
        ];
      };
    };
}
